
package control.pack.controlfield;

import java.util.EventListener;

import org.eclipse.scout.commons.EventListenerList;
import org.eclipse.scout.commons.annotations.ConfigOperation;
import org.eclipse.scout.commons.annotations.Order;
import org.eclipse.scout.commons.exception.ProcessingException;
import org.eclipse.scout.rt.client.ui.form.fields.AbstractFormField;
import org.eclipse.scout.rt.shared.services.common.exceptionhandler.IExceptionHandlerService;
import org.eclipse.scout.service.SERVICES;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

public abstract class AbstractControlField extends AbstractFormField implements IControlField {
	
	private class ControlFieldUIFacade implements IControlFieldUIFacade {
		
		@Override
		public void fireClick() {
		
			AbstractControlField.this.fireClick();
		}
	}
	
	private Composite               parent;
	
	private IControlFieldUIFacade   controlFieldUIFacade;
	
	private final EventListenerList listenerList = new EventListenerList();
	
	@Override
	public final void addControlFieldListener(final IControlFieldListener listener) {
	
		this.listenerList.add(IControlFieldListener.class, listener);
	}
	
	@Override
	public final Control getControl() {
	
		return (Control) this.propertySupport.getProperty(IControlField.PROPERTY_CONTROL);
	}
	
	@Override
	public final IControlFieldUIFacade getControlFieldUIFacade() {
	
		return this.controlFieldUIFacade;
	}
	
	public Composite getParent() {
	
		return this.parent;
	}
	
	@Override
	public final void removeControlFieldListener(final IControlFieldListener listener) {
	
		this.listenerList.remove(IControlFieldListener.class, listener);
	}
	
	@Override
	public final void setControl(final Control control) {
	
		this.propertySupport.setProperty(IControlField.PROPERTY_CONTROL, control);
	}
	
	@Override
	public final void setControlFieldUIFacade(final IControlFieldUIFacade controlFieldUIFacade) {
	
		this.controlFieldUIFacade = controlFieldUIFacade;
	}
	
	public void setParent(final Composite parent) {
	
		this.parent = parent;
	}
	
	@ConfigOperation
	@Order(10)
	protected void execClicked(final ControlFieldEvent e) throws ProcessingException {
	
	}
	
	@Override
	protected final void initConfig() {
	
		this.controlFieldUIFacade = new ControlFieldUIFacade();
		super.initConfig();
	}
	
	private void fireClick() {
	
		final ControlFieldEvent event = new ControlFieldEvent(this);
		try {
			this.execClicked(event);
		}
		catch (final ProcessingException pe) {
			SERVICES.getService(IExceptionHandlerService.class).handleException(pe);
		}
		catch (final Throwable t) {
			SERVICES.getService(IExceptionHandlerService.class).handleException(new ProcessingException("Unexpected", t));
		}
		this.fireControlFieldEventInternal(event);
	}
	
	private void fireControlFieldEventInternal(final ControlFieldEvent event) {
	
		final EventListener[] listeners = this.listenerList.getListeners(IControlFieldListener.class);
		if (listeners != null) {
			for (final EventListener element : listeners) {
				((IControlFieldListener) element).handleControlFieldEvent(event);
			}
		}
	}
}
